//
//  FooAlertView.m
//  GJAlertDemo
//
//  Created by Greg Jaskiewicz on 07/04/2012.
//  Copyright (c) 2012 K4Lab. All rights reserved.
//

#import "FooAlertView.h"
#import "UIImage+imageWithColour.h"
#import <QuartzCore/CALayer.h>

const size_t sBackgroundCapHeight = 30;
const size_t sBorder              = 10;
const size_t sButtonHeight        = 45;
const size_t sBounce              = 10;

@interface FooAlertView ()

@property (nonatomic) CGFloat alertWindowWidth;
@property (nonatomic, strong) UIWindow *window;
@property (nonatomic, strong) NSString *title;
@property (nonatomic, strong) UIImage *background;
@property (nonatomic, strong, readonly) UIFont *titleFont;
@property (nonatomic, strong, readonly) UIFont *buttonFont;
@property (nonatomic, strong, readonly) NSMutableArray *buttons;
@property (nonatomic, strong) NSMutableArray *buttonCaption;

@end

@implementation FooAlertView

/* 
 PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP
 Plazy getters/initialisers 
 PPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPPP
 */

- (NSMutableArray*)buttonCaption
{
  if (!_buttonCaption)
  {
    _buttonCaption = [NSMutableArray array];
  }
  
  return _buttonCaption;
}


/* 
 iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii
 initializers 
 iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii
 */

+ (FooAlertView *)fooAlertViewWithTitle:(NSString *)title
{
  FooAlertView* _alert;
  
  _alert = [[FooAlertView alloc] initWitTitle:title];
  return _alert;
}

- (void)initialize
{
  //[self.window setBackgroundColor:[UIColor colorWithWhite:0.0 alpha:0.5]];
  self.alertWindowWidth = [self.window bounds].size.width - 100;
  
  self.animate = YES;
  self.animationSpeed = 1;

  _window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
  _title = @"";
  _background = [UIImage imageNamed:@"alert-window.png"];
  _background = [_background stretchableImageWithLeftCapWidth:0 topCapHeight:sBackgroundCapHeight];
  _titleFont = [UIFont boldSystemFontOfSize:20];
  _buttonFont = [UIFont boldSystemFontOfSize:18];
  _buttons = [NSMutableArray array];
}

- (id)initWitTitle:(NSString*)title
{
  self = [super init];
  if (self)
  {
    [self initialize];
    _title = title;

    CGSize size = [title sizeWithAttributes:@{NSFontAttributeName: self.titleFont}];
    self.alertWindowWidth = MAX(size.width, self.alertWindowWidth);
  }

  return self;
}

- (id)initWithFrame:(CGRect)frame
{
  self = [super initWithFrame:frame];
  if (self) 
  {
    [self initialize];
  }

  return self;
}

- (NSString*)buttonCaptionFor:(NSInteger)index
{
  if (index >= 0 || index < [self.buttonCaption count])
  {
    NSString *caption = self.buttonCaption[index];
    return caption;
  }
  
  return @"";
}

- (NSInteger)addButton:(NSString*)caption withColour:(UIColor*)colour
{
  if (!caption || !colour)
  {
    return -1;
  }
  
  UIImage *image = [UIImage imageNamed:[NSString stringWithFormat:@"alert-gray-button.png"] withColor:colour];
  image = [image stretchableImageWithLeftCapWidth:(int)(image.size.width)>>1 topCapHeight:0];
  
  UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
  button.titleLabel.font = self.buttonFont;
  button.titleLabel.minimumScaleFactor = 0.5;
  button.titleLabel.adjustsFontSizeToFitWidth = YES;
  button.titleLabel.textAlignment = NSTextAlignmentCenter;
  button.titleLabel.shadowOffset = CGSizeMake(0, -1);
  button.backgroundColor = [UIColor clearColor];
  button.tag = [self.buttons count];
  
  [button setBackgroundImage:image forState:UIControlStateNormal];
  [button setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
  [button setTitleShadowColor:[UIColor blackColor] forState:UIControlStateNormal];
  [button setTitle:caption forState:UIControlStateNormal];
  button.accessibilityLabel = caption;
  [button addTarget:self action:@selector(buttonClicked:) forControlEvents:UIControlEventTouchUpInside];
  
  [self.buttons addObject:button];
  [self.buttonCaption addObject:caption];
  
  /* calculate max width required for the alarm view */
  CGSize size = [caption sizeWithAttributes:@{NSFontAttributeName: self.buttonFont}];
  self.alertWindowWidth = MAX(size.width, self.alertWindowWidth);
  
  return [self.buttons count]-1; 
}

- (void)buttonClicked:(UIButton*)sender 
{
  /* Run the button's block */
  int buttonIndex = [sender tag];
  if (buttonIndex >= 0 && buttonIndex < [self.buttons count])
  {
    [self.delegate fooAlertView:self buttonIndex:buttonIndex];
  }
  
  if (self.animate)
  {
    [UIView animateWithDuration:0.02*self.animationSpeed
                          delay:0.0
                        options:UIViewAnimationOptionCurveEaseOut
                     animations:
     ^{
       self.alpha = 0.5f;
       self.transform = CGAffineTransformMakeScale(1.1, 1.1); 
     } 
     
                     completion:
     ^(BOOL finished) {
       /* when done with the first animation, fire second one to get us back to x1 size */
       [UIView animateWithDuration:0.05*self.animationSpeed 
                             delay:0.0
                           options:UIViewAnimationOptionCurveEaseOut
                        animations:
        ^{
          self.transform = CGAffineTransformMakeScale(0.1, 0.1); 
          self.alpha = 0.0f;
        }
                        completion:
        ^(BOOL finished) {
          [self removeFromSuperview];
        }]; 
     }];
  }
  else 
  {
    [self removeFromSuperview];
  }
}


- (void)show
{ 
  // We need to add it to the window, which we can get from the delegate
  [self.window addSubview:self];
  [self.window makeKeyAndVisible];
  
  // Make sure the alert covers the whole window
  self.frame  = self.window.frame;
  self.center = self.window.center;
  
  /* now add the background, with rounded corners and white frame */
  UIImageView *modalBackground = [[UIImageView alloc] initWithFrame:self.bounds];
  modalBackground.image = self.background;
  
  /* calculate height of the whole thing */
  CGFloat height = [self.buttons count]*(sButtonHeight+sBounce)+sButtonHeight;
  
  self.alertWindowWidth += sBorder*2;
  
  /* add the title */
  UILabel *labelView = [[UILabel alloc] initWithFrame:CGRectMake(sBorder, sBorder, self.alertWindowWidth, sButtonHeight)];
  labelView.font = self.titleFont;
  labelView.numberOfLines = 0;
  labelView.lineBreakMode = NSLineBreakByWordWrapping;
  labelView.textColor = [UIColor whiteColor];
  labelView.backgroundColor = [UIColor clearColor];
  labelView.textAlignment = NSTextAlignmentCenter;
  labelView.shadowColor = [UIColor grayColor];
  labelView.shadowOffset = CGSizeMake(0, -1);
  labelView.text = _title;
  [modalBackground addSubview:labelView];
  
  /* frame */
  [modalBackground.layer setBorderColor: [[UIColor grayColor] CGColor]];
  [modalBackground.layer setBorderWidth: 2.5];
  
  /* rounded corners */
  [modalBackground.layer setMasksToBounds:YES];
  [modalBackground.layer setCornerRadius:10.0];
  [modalBackground.layer setOpacity:0.83];
  
  /* calculate all the sizes */
  CGRect frame = self.frame;
  frame.origin.y = self.bounds.size.height;
  frame.size.height = height + sBounce;
  frame.size.width = self.alertWindowWidth + sBorder*2;
  modalBackground.frame = frame;
  
  CGPoint mycenter = self.center;
  mycenter.y = [[self.window screen] bounds].size.height / 2;
  mycenter.x = [[self.window screen] bounds].size.width  / 2;
  
  modalBackground.contentMode = UIViewContentModeScaleToFill;
  
  modalBackground.frame = frame;
  modalBackground.center = mycenter;
  
  /* add buttons here, after modalBackground frame is calculated */  
  for(UIButton* button in self.buttons)
  {
    CGRect _frame = CGRectMake(modalBackground.frame.origin.x+sBorder, 
                               modalBackground.frame.origin.y+([button tag]+1)*(sButtonHeight+sBounce), 
                               self.alertWindowWidth,
                               sButtonHeight);
    
    [button setFrame:_frame];
    [button setEnabled:YES];
    
    [self addSubview:button];
  }

  [self insertSubview:modalBackground atIndex:0];  

  if (self.animate)
  {
    /* initial animation condition */
    self.transform = CGAffineTransformMakeScale(0.1, 0.1); 
    self.alpha = 0.0f;
    
    [UIView animateWithDuration:0.05*self.animationSpeed
                          delay:0.0
                        options:UIViewAnimationOptionCurveEaseOut
                     animations:
     ^{
       self.alpha = 1.0f;
       self.transform = CGAffineTransformMakeScale(1.1, 1.1); 
       //[self setBackgroundColor:[UIColor colorWithRed:0.1 green:0.1 blue:0.1 alpha:0.4f]];
     } 
     
                     completion:
     ^(BOOL finished) {
       /* when done with the first animation, fire second one to get us back to x1 size */
       [UIView animateWithDuration:0.02*self.animationSpeed animations:^{
         self.transform = CGAffineTransformMakeScale(1.0, 1.0); 
       }];
     }];
  }
}

@end
