//
//  NSImageWithColour.m
//  GJAlertDemo
//
//  Created by Greg Jaskiewicz on 07/04/2012.
//  Copyright (c) 2012 K4Lab. All rights reserved.
//

#import "UIImage+imageWithColour.h"

@implementation UIImage (imageWithColour)

+ (UIImage *)imageNamed:(NSString *)name withColor:(UIColor *)color
{
  if (!color) {
    return nil;
  }

  UIImage *img = [UIImage imageNamed:name];
  
  if (!img)
  {
    return nil;
  }
  
  // begin a new image context, to draw our colored image onto
  UIGraphicsBeginImageContext(img.size);
  
  // get a reference to that context we created
  CGContextRef context = UIGraphicsGetCurrentContext();
  
  // set the fill color
  [color setFill];
  
  // translate/flip the graphics context (for transforming from CG* coords to UI* coords
  CGContextTranslateCTM(context, 0, img.size.height);
  CGContextScaleCTM(context, 1.0, -1.0);
  
  // set the blend mode to color burn, and the original image
  CGContextSetBlendMode(context, kCGBlendModeColorBurn);
  CGRect rect = CGRectMake(0, 0, img.size.width, img.size.height);
  CGContextDrawImage(context, rect, img.CGImage);
  
  // set a mask that matches the shape of the image, then draw (color burn) a colored rectangle
  CGContextClipToMask(context, rect, img.CGImage);
  CGContextAddRect(context, rect);
  CGContextDrawPath(context,kCGPathFill);
  
  // generate a new UIImage from the graphics context we drew onto
  UIImage *coloredImg = UIGraphicsGetImageFromCurrentImageContext();
  UIGraphicsEndImageContext();
  
  //return the color-burned image
  return coloredImg;
}

@end;
